import uuid
from typing import Optional, Union

from rcs_pydantic import enums

from . import scheme


class RcsMessage:
    def __init__(
        self,
        message_info: scheme.MessageInfo,
        body: Union[
            scheme.RcsSMSBody,
            scheme.RcsLMSBody,
            scheme.RcsMMSBody,
            scheme.RcsCHATBody,
            scheme.RcsSMSCarouselBody,
            scheme.RcsLMSCarouselBody,
            scheme.RcsMMSCarouselBody,
            scheme.RcsCHATCarouselBody,
            dict,
        ],
        agency_id: Optional[str] = None,
        message_base_id: Union[enums.MessageEnum, enums.RCSMessageEnum] = enums.MessageEnum.SMS,
        service_type: enums.ServiceTypeEnum = enums.ServiceTypeEnum.SMS,
        expiry_option: Optional[enums.ExpiryOptionEnum] = None,
        header: enums.HeaderEnum = enums.HeaderEnum.NOT_ADVERTISE,
        footer: Optional[str] = None,
        cdr_id: Optional[str] = None,
        copy_allowed: Optional[bool] = None,
        buttons: Optional[list] = None,
        chips: Optional[list] = None,
        legacy: Optional[scheme.LegacyInfo] = None,
        message_group_id: Optional[str] = None,
    ):
        self.message_info = message_info
        self.agency_id = agency_id
        self.message_base_id = message_base_id
        self.service_type = service_type
        self.expiry_option = expiry_option
        self.header = header
        self.footer = footer
        self.cdr_id = cdr_id
        self.copy_allowed = copy_allowed
        self.body = body
        self.buttons = buttons
        self.chips = chips
        self.legacy = legacy
        self.message_group_id = message_group_id
        self.send_info = self.make_send_info(
            common=scheme.CommonInfo(**self.make_common_info(message_info)),
            rcs=scheme.RcsInfo(**self.make_rcs_info(message_info)),
        )

    def make_send_info(self, common, rcs):
        if self.legacy:
            return scheme.SendInfo(
                common=common,
                rcs=rcs,
                legacy=self.legacy,
            )

        else:
            return scheme.SendInfo(
                common=common,
                rcs=rcs,
            )

    def make_common_info(self, message_info: scheme.MessageInfo) -> dict:
        if self.legacy:
            msg_service_type = enums.MessageServiceTypeEnum.RCS_LEGACY
        else:
            msg_service_type = enums.MessageServiceTypeEnum.RCS

        common = {
            "msgId": str(uuid.uuid4()),
            "userContact": message_info.userContact,
            "scheduleType": 0,
            "msgServiceType": msg_service_type,
        }

        if self.message_group_id:
            common["msgGroupId"] = self.message_group_id

        return scheme.CommonInfo(**common).dict(exclude_unset=True)

    def make_rcs_info(self, message_info: scheme.MessageInfo) -> dict:
        rcs_info = scheme.RcsInfo(
            chatbotId=message_info.chatbotId,
            messagebaseId=self.message_base_id,
            serviceType=self.service_type,
            header=self.header,
            body=self.body,
        )
        if self.agency_id:
            rcs_info.agencyId = self.agency_id
        if self.expiry_option:
            rcs_info.expiryOption = self.expiry_option
        if self.service_type == enums.ServiceTypeEnum.CHAT:
            rcs_info.expiryOption = enums.ExpiryOptionEnum.AFTER_SETTING_TIMES
        if self.footer:
            rcs_info.footer = self.footer
        if self.cdr_id:
            rcs_info.cdrId = self.cdr_id
        if self.copy_allowed:
            rcs_info.copyAllowed = self.copy_allowed
        if self.buttons:
            rcs_info.buttons = self.buttons
        if self.chips:
            rcs_info.chipList = self.chips
        if message_info.replyId:
            rcs_info.replyId = message_info.replyId
        return rcs_info.dict(exclude_unset=True)

    def send(self):
        self.send_info



from enum import Enum, IntEnum


class MessageServiceTypeEnum(Enum):
    RCS: str = "rcs"
    RCS_LEGACY: str = "rcs, legacy"  # rcs 부달시, xMS로fallback전송


class ServiceTypeEnum(Enum):
    SMS: str = "RCSSMS"
    LMS: str = "RCSLMS"
    MMS: str = "RCSMMS"
    TMPL: str = "RCSTMPL"
    CHAT: str = "RCSCHAT"


class LegacyServiceTypeEnum(Enum):
    SMS: str = "SMS"
    LMS: str = "LMS"
    MMS: str = "MMS"


class ScheduleTypeEnum(IntEnum):
    IMMEDIATE: int = 0


class ExpiryOptionEnum(IntEnum):
    AFTER_THREE_DAYS: int = 1
    AFTER_SETTING_TIMES: int = 2


class HeaderEnum(Enum):
    """
    헤더 광고에 대한 정의
    0: 광고 넣지않음
    1: 광고 넣음
    """

    NOT_ADVERTISE: str = "0"
    ADVERTISE: str = "1"


class ActionEnum(Enum):
    URL_ACTION: str = "urlAction"  # 단말기에 기본 웹 브라우저로 설정된 앱을 통해서, 웹페이지로 이동할 수있습니다
    LOCAL_BROWSER_ACTION: str = "localBrowserAction"  # 단말기의 메시지 앱 내부 브라우저를 통해 웹페이지로 이동할 수 있습니다
    MAP_ACTION: str = "mapAction"  # 미리 지정된 위치를 보여주거나 사용자의 현재 위치를 서버로 전송 할 수 있습니다.
    CALENDAR_ACTION: str = "calendarAction"  # 사용자의 캘린더에 특정 일정을 등록 할 수 있습니다.
    CLIPBOARD_ACTION: str = "clipboardAction"  # 특정 문구를 사용자 단말이 자동으로 복사 할 수 있게 합니다
    COMPOSE_ACTION: str = "composeAction"  # 다른 번호로 메시지를 보낼 수 있도록 대화방을 엽니다.
    DIALER_ACTION: str = "dialerAction"  # 특정 전화번호로 전화를 걸 수 있습니다.


class MessageStatusEnum(Enum):
    SUCCESS: str = "success"
    FAIL: str = "fail"


class MnoInfoEnum(Enum):
    KT: str = "KT"
    SKT: str = "SKT"
    LGU: str = "LGU"


class BillEnum(IntEnum):
    FREE: int = 0
    CHARGE: int = 1


class EventTypeEnum(Enum):
    MESSAGE: str = "message"
    RESPONSE: str = "response"
    NEW_USER: str = "newUser"


class RCSMessageEnum(Enum):
    SMS: str = "SCS00000"  # 기본 말풍선 (SMS)
    LMS: str = "SCL00000"  # 텍스트 카드 (LMS)
    CAROUSEL_MEDIUM_2: str = "CCwMhM0200"  # 슬라이드형(Medium, 2장)
    CAROUSEL_MEDIUM_3: str = "CCwMhM0300"  # 슬라이드형(Medium, 3장)
    CAROUSEL_MEDIUM_4: str = "CCwMhM0400"  # 슬라이드형(Medium, 4장)
    CAROUSEL_MEDIUM_5: str = "CCwMhM0500"  # 슬라이드형(Medium, 5장)
    CAROUSEL_MEDIUM_6: str = "CCwMhM0600"  # 슬라이드형(Medium, 6장)
    CAROUSEL_SMALL_2: str = "CCwShS0200"  # 슬라이드형(Small, 2장)
    CAROUSEL_SMALL_3: str = "CCwShS0300"  # 슬라이드형(Small, 3장)
    CAROUSEL_SMALL_4: str = "CCwShS0400"  # 슬라이드형(Small, 4장)
    CAROUSEL_SMALL_5: str = "CCwShS0500"  # 슬라이드형(Small, 5장)
    CAROUSEL_SMALL_6: str = "CCwShS0600"  # 슬라이드형(Small, 6장)
    STANDALONE_MEDIA_TOP_TALL: str = "SCwThT00"  # 세로형(Tall)
    STANDALONE_MEDIA_TOP_MEDIUM: str = "SCwThM00"  # 세로형(Medium)


class MessageEnum(Enum):
    SMS: str = "SS000000"  # 기본 말풍선 (SMS)
    LMS: str = "SL000000"  # 텍스트 카드 (LMS)
    STANDALONE_MEDIA_TOP_TALL: str = "SMwThT00"  # 세로형(Tall)
    STANDALONE_MEDIA_TOP_MEDIUM: str = "SMwThM00"  # 세로형(Medium)
    CAROUSEL_MEDIUM_3: str = "CMwMhM0300"  # 슬라이드형(Medium, 3장)
    CAROUSEL_MEDIUM_4: str = "CMwMhM0400"  # 슬라이드형(Medium, 4장)
    CAROUSEL_MEDIUM_5: str = "CMwMhM0500"  # 슬라이드형(Medium, 5장)
    CAROUSEL_MEDIUM_6: str = "CMwMhM0600"  # 슬라이드형(Medium, 6장)
    CAROUSEL_SMALL_3: str = "CMwShS0300"  # 슬라이드형(Small, 3장)
    CAROUSEL_SMALL_4: str = "CMwShS0400"  # 슬라이드형(Small, 4장)
    CAROUSEL_SMALL_5: str = "CMwShS0500"  # 슬라이드형(Small, 5장)
    CAROUSEL_SMALL_6: str = "CMwShS0600"  # 슬라이드형(Small, 6장)


class FileUsageTypeEnum(Enum):
    SEND: str = "send"


class FileUsageServiceEnum(Enum):
    RCS: str = "RCS"
    MMS: str = "MMS"


class FileStatusEnum(Enum):
    READY: str = "ready"
    EXPIRED: str = "expired"
